var Context = require('./context');
var EndpointContext = require('./endpoint-context');
var Error = require('./error');
var MetaContext = require('./meta-context');

var Datapoint = function () {
    /**
     * @type {Context[]}
     */
    this.contextCollection = [];
    /**
     * @type {Context[]}
     */
    this.inputCollection = [];
    /**
     * @type {Error[]}
     */
    this.errorCollection = [];
    /**
     * @type {Context[]}
     */
    this.endpointCollection = [];
    /**
     * @type {MetaCollection[]}
     */
    this.metaContextCollection = [];
};

/**
 * @param {Context} context
 */
Datapoint.prototype.addContext = function (context) {
    if (context instanceof Context) {

        this.contextCollection.push(context);
    }
};

/**
 * @param {Context} context
 */
Datapoint.prototype.addEndpointContext = function (context) {
    if (context instanceof Context) {
        this.endpointCollection.push(context);
    }
};

/**
 * @param {Context} input
 */
Datapoint.prototype.addInput = function (input) {
   // if (input instanceof Context) {
        this.inputCollection.push(input);
   // }
};

/**
 * @param {Error} error
 */
Datapoint.prototype.addError = function (error) {
    this.errorCollection.push(error);
};

/**
 * @param {MetaContext} metaContext
 */
Datapoint.prototype.addMetaContext = function(metaContext){

    if(metaContext instanceof MetaContext){
        this.metaContextCollection.push(metaContext);
    }
}
/**
 * @param {boolean, optional} prettyPrint
 */
Datapoint.prototype.toJSON = function (prettyPrint) {
    if (prettyPrint === undefined) prettyPrint = false;
    return JSON.stringify(this.toArray(), null, !prettyPrint ? null : 2);
};

Datapoint.prototype.toArray = function () {
    var all = {};
    all[Context.NAME] = {};
    all[EndpointContext.NAME] = {};
    all['input'] = {};
    all['errors'] = [];

    this.contextCollection.forEach(function (context) {
        all[Context.NAME][context.name] = context.value;
    });

    this.endpointCollection.forEach(function (context) {
        all[EndpointContext.NAME][context.name] = context.value;
    });

    this.inputCollection.forEach(function (input) {
        all.input[input.name] = input.value;
    });

    this.errorCollection.forEach(function (error) {
        all.errors.push(error);
    });

    //TODO metaContextCollectionK

    return all;
};

module.exports = Datapoint;